@@ -1,4 +1,3 @@
-from datetime import datetime
 from typing import Optional
 
 from evernote_backup.cli_app_auth_oauth import evernote_login_oauth
@@ -6,6 +5,7 @@ from evernote_backup.cli_app_auth_password import evernote_login_password
 from evernote_backup.cli_app_util import ProgramTerminatedError, logger
 from evernote_backup.evernote_client_sync import EvernoteClientSync
 from evernote_backup.evernote_client_util import EvernoteAuthError
+from evernote_backup.token_util import get_token_expiration_date
 
 
 def get_sync_client(
@@ -55,11 +55,3 @@ def get_auth_token(
         backend,
         network_retry_count,
     )
-
-
-def get_token_expiration_date(token: str) -> datetime:
-    hex_base = 16
-    token_parts = token.split(":")
-    token_expire_ts = int(token_parts[2][2:], base=hex_base) // 1000
-
-    return datetime.utcfromtimestamp(token_expire_ts)
