@@ -1,14 +1,15 @@
 import base64
 import time
+from typing import Optional
 
 
-def fmt_time(timestamp):
+def fmt_time(timestamp: Optional[int]) -> Optional[str]:
     if timestamp is None:
         return timestamp
     return time.strftime("%Y%m%dT%H%M%SZ", time.gmtime(timestamp // 1000))
 
 
-def fmt_binary(binary_data):
+def fmt_binary(binary_data: bytes) -> str:
     slice_width = 120
     return (
         "\n"
@@ -17,7 +18,7 @@ def fmt_binary(binary_data):
     )
 
 
-def fmt_content(content_body):
+def fmt_content(content_body: Optional[str]) -> Optional[str]:
     if content_body is None:
         return content_body
 
@@ -37,7 +38,7 @@ def fmt_content(content_body):
     )
 
 
-def _slice_str(text, width):
+def _slice_str(text: str, width: int) -> str:
     slice_steps = range(0, len(text), width)
     slice_lines = (text[i : i + width] for i in slice_steps)
     return "\n".join(slice_lines)
