@@ -5,20 +5,32 @@ import lzma
 import os
 import pickle
 import sqlite3
-from typing import Iterable, Iterator, Tuple, Union
+from typing import Iterable, Iterator, NamedTuple, Optional, Tuple, Union
 
-from evernote.edam.type.ttypes import Note, Notebook
+from evernote.edam.type.ttypes import LinkedNotebook, Note, Notebook
 
 from evernote_backup.config import CURRENT_DB_VERSION
 from evernote_backup.log_util import log_format_note, log_format_notebook
 
 logger = logging.getLogger(__name__)
 
+
+class NoteForSync(NamedTuple):
+    guid: str
+    title: str
+    linked_notebook_guid: Optional[str]
+
+
 DB_SCHEMA = """CREATE TABLE IF NOT EXISTS notebooks(
                         guid TEXT PRIMARY KEY,
                         name TEXT,
                         stack TEXT
                     );
+                    CREATE TABLE IF NOT EXISTS notebooks_linked(
+                        guid TEXT PRIMARY KEY,
+                        notebook_guid TEXT,
+                        usn INT DEFAULT 0
+                    );
                     CREATE TABLE IF NOT EXISTS notes(
                         guid TEXT PRIMARY KEY,
                         title TEXT,
@@ -33,7 +45,10 @@ DB_SCHEMA = """CREATE TABLE IF NOT EXISTS notebooks(
                     CREATE INDEX IF NOT EXISTS idx_notes
                      ON notes(notebook_guid, is_active);
                     CREATE INDEX IF NOT EXISTS idx_notes_title
-                     ON notes(title COLLATE NOCASE);"""
+                     ON notes(title COLLATE NOCASE);
+                    CREATE INDEX IF NOT EXISTS idx_notebooks_linked
+                     ON notebooks_linked(guid, notebook_guid);
+"""
 
 
 class DatabaseResyncRequiredError(Exception):
@@ -102,6 +117,20 @@ class SqliteStorage(object):
                     " ON notes(title COLLATE NOCASE);"
                 )
 
+        if db_version < 4:
+            with self.db as con3:
+                con3.execute(
+                    "CREATE TABLE IF NOT EXISTS notebooks_linked("
+                    " guid TEXT PRIMARY KEY,"
+                    " notebook_guid TEXT,"
+                    " usn INT DEFAULT 0"
+                    " );"
+                )
+                con3.execute(
+                    "CREATE INDEX IF NOT EXISTS idx_notebooks_linked"
+                    " ON notebooks_linked(guid, notebook_guid);"
+                )
+
         self.config.set_config_value("DB_VERSION", str(CURRENT_DB_VERSION))
 
         if need_resync:
@@ -109,7 +138,7 @@ class SqliteStorage(object):
             raise DatabaseResyncRequiredError
 
 
-class NoteBookStorage(SqliteStorage):
+class NoteBookStorage(SqliteStorage):  # noqa: WPS214
     def add_notebooks(self, notebooks: Iterable[Notebook]) -> None:
         if logger.getEffectiveLevel() == logging.DEBUG:  # pragma: no cover
             for nb in notebooks:
@@ -151,8 +180,72 @@ class NoteBookStorage(SqliteStorage):
         with self.db as con:
             con.executemany("delete from notebooks where guid=?", ((g,) for g in guids))
 
+    def add_linked_notebook(
+        self, l_notebook: LinkedNotebook, notebook: Notebook
+    ) -> None:
+        if logger.getEffectiveLevel() == logging.DEBUG:  # pragma: no cover
+            logger.debug(
+                f"Adding/updating linked notebook {l_notebook.shareName}"
+                " [{l_notebook.guid}] -> [{notebook.guid}]"
+            )
 
-class NoteStorage(SqliteStorage):
+        with self.db as con:
+            con.execute(
+                "replace into notebooks_linked(guid, notebook_guid) values (?, ?)",
+                (l_notebook.guid, notebook.guid),
+            )
+
+    def get_notebook_by_linked_guid(self, l_notebook_guid: str) -> Optional[Notebook]:
+        with self.db as con:
+            cur = con.execute(
+                "select notebooks.guid, notebooks.name, notebooks.stack"
+                " from notebooks_linked"
+                " join notebooks"
+                " on notebooks.guid=notebooks_linked.notebook_guid"
+                " where notebooks_linked.guid=?",
+                (l_notebook_guid,),
+            )
+
+            row = cur.fetchone()
+
+            if row is None:
+                return None
+
+            return Notebook(
+                guid=row["guid"],
+                name=row["name"],
+                stack=row["stack"],
+            )
+
+    def get_linked_notebook_usn(self, l_notebook_guid: str) -> int:
+        with self.db as con:
+            cur = con.execute(
+                "select usn from notebooks_linked where guid=?",
+                (l_notebook_guid,),
+            )
+
+            res = cur.fetchone()
+
+            if res is None:
+                return 0
+
+            return int(res[0])
+
+    def set_linked_notebook_usn(self, l_notebook_guid: str, usn: int) -> None:
+        with self.db as con:
+            con.execute(
+                "update notebooks_linked set usn=? where guid=?",
+                (usn, l_notebook_guid),
+            )
+
+    def expunge_linked_notebooks(self, guids: Iterable[str]) -> None:
+        with self.db as con:
+            con.executemany(
+                "delete from notebooks_linked where guid=?", ((g,) for g in guids)
+            )
+
+
+class NoteStorage(SqliteStorage):  # noqa: WPS214
     def add_notes_for_sync(self, notes: Iterable[Note]) -> None:
         if logger.getEffectiveLevel() == logging.DEBUG:  # pragma: no cover
             for note in notes:
@@ -161,8 +254,8 @@ class NoteStorage(SqliteStorage):
 
         with self.db as con:
             con.executemany(
-                "replace into notes(guid, title) values (?, ?)",
-                ((n.guid, n.title) for n in notes),
+                "replace into notes(guid, title, notebook_guid) values (?, ?, ?)",
+                ((n.guid, n.title, n.notebookGuid) for n in notes),
             )
 
     def add_note(self, note: Note) -> None:
@@ -206,11 +299,24 @@ class NoteStorage(SqliteStorage):
 
             yield from (pickle.loads(lzma.decompress(row["raw_note"])) for row in cur)
 
-    def get_notes_for_sync(self) -> Tuple[Tuple[str, str], ...]:
+    def get_notes_for_sync(self) -> Tuple[NoteForSync, ...]:
         with self.db as con:
-            cur = con.execute("select guid, title from notes where raw_note is NULL")
+            cur = con.execute(
+                "select notes.guid, title, notebooks_linked.guid as l_notebook"
+                " from notes"
+                " left join notebooks_linked"
+                " using (notebook_guid)"
+                " where raw_note is NULL"
+            )
 
-            notes = ((row["guid"], row["title"]) for row in cur.fetchall())
+            notes = (
+                NoteForSync(
+                    guid=row["guid"],
+                    title=row["title"],
+                    linked_notebook_guid=row["l_notebook"],
+                )
+                for row in cur.fetchall()
+            )
 
             return tuple(notes)
 
@@ -218,6 +324,10 @@ class NoteStorage(SqliteStorage):
         with self.db as con:
             con.executemany("delete from notes where guid=?", ((g,) for g in guids))
 
+    def expunge_notes_by_notebook(self, notebook_guid: str) -> None:
+        with self.db as con:
+            con.execute("delete from notes where notebook_guid=?", (notebook_guid,))
+
     def get_notes_count(self, is_active: bool = True) -> int:
         with self.db as con:
             cur = con.execute(
