@@ -3,14 +3,15 @@
 import logging
 import threading
 from concurrent.futures import ThreadPoolExecutor, as_completed, wait
-from typing import Any, Iterable, List, Tuple
+from typing import Any, Dict, Iterable, List, Tuple
 
 from click import progressbar
-from evernote.edam.type.ttypes import Note
+from evernote.edam.notestore.ttypes import SyncChunk
+from evernote.edam.type.ttypes import LinkedNotebook, Note
 
 from evernote_backup.cli_app_util import chunks, get_progress_output
 from evernote_backup.evernote_client_sync import EvernoteClientSync
-from evernote_backup.note_storage import SqliteStorage
+from evernote_backup.note_storage import NoteForSync, SqliteStorage
 
 logger = logging.getLogger(__name__)
 
@@ -46,25 +47,36 @@ class NoteClientWorker(object):
 
         self._thread_data = threading.local()
 
-    def __call__(self, note_id: str) -> Note:
+    def __call__(self, note_id: str, token: str = None) -> Note:
         if self.stop:
             raise WorkerStopException
 
+        if token is None:
+            token = self.token
+
         try:
-            note_client = self._thread_data.note_client
-        except AttributeError:
+            note_client = self.clients[token]
+        except KeyError:
             note_client = EvernoteClientSync(
-                token=self.token,
+                token=token,
                 backend=self.backend,
                 network_error_retry_count=self.network_error_retry_count,
                 max_chunk_results=self.max_chunk_results,
             )
-            self._thread_data.note_client = note_client
+            self.clients[token] = note_client
 
         return note_client.get_note(note_id)
 
+    @property
+    def clients(self) -> Any:
+        try:
+            return self._thread_data.note_clients
+        except AttributeError:
+            self._thread_data.note_clients = {}
+            return self._thread_data.note_clients
 
-class NoteSynchronizer(object):
+
+class NoteSynchronizer(object):  # noqa: WPS214
     def __init__(
         self,
         note_client: EvernoteClientSync,
@@ -75,6 +87,7 @@ class NoteSynchronizer(object):
         self._count_updated_notes = 0
 
         self._count_expunged_notebooks = 0
+        self._count_expunged_linked_notebooks = 0
         self._count_expunged_notes = 0
 
         self.note_client = note_client
@@ -87,23 +100,27 @@ class NoteSynchronizer(object):
             self.note_client.network_error_retry_count,
             self.note_client.max_chunk_results,
         )
+        self.linked_notebooks_auth: Dict[str, str] = {}
 
     def sync(self) -> None:
         self._raise_on_wrong_user()
 
-        remote_usn = self.note_client.get_remote_usn()
-        current_usn = int(self.storage.config.get_config_value("USN"))
+        logger.info("Syncing user notebooks...")
 
-        if remote_usn == current_usn:
-            logger.info("Nothing to sync, current database is up to date!")
-        else:
-            logger.info("Syncing latest changes...")
-            self._sync_chunks(current_usn, remote_usn)
+        self._sync_chunks()
+
+        if self.note_client.linked_notebooks:
+            logger.info("Syncing linked notebooks...")
+
+            for l_notebook in self.note_client.linked_notebooks.values():
+                self._sync_linked_notebook(l_notebook)
 
         notes_to_sync = self.storage.notes.get_notes_for_sync()
 
         if notes_to_sync:
             logger.info("{0} notes to download...".format(len(notes_to_sync)))
+
+            self._authorize_linked_notebooks_for_notes(notes_to_sync)
             self._download_scheduled_notes(notes_to_sync)
 
             self._count_updated_notes = len(notes_to_sync)
@@ -112,12 +129,32 @@ class NoteSynchronizer(object):
             f"Updated or added notebooks: {self._count_updated_notebooks}",
             f"Updated or added notes: {self._count_updated_notes}",
             f"Expunged notebooks: {self._count_expunged_notebooks}",
+            f"Expunged linked notebooks: {self._count_expunged_linked_notebooks}",
             f"Expunged notes: {self._count_expunged_notes}",
         ]
 
         for msg in report:
             logger.info(msg)
 
+    def _authorize_linked_notebooks_for_notes(
+        self, notes_to_sync: Tuple[NoteForSync, ...]
+    ) -> None:
+        linked_notebooks = {
+            n.linked_notebook_guid for n in notes_to_sync if n.linked_notebook_guid
+        }
+
+        if linked_notebooks:
+            logger.debug(
+                "Requesting access to {0} linked notebooks...".format(
+                    len(linked_notebooks)
+                )
+            )
+
+            self.linked_notebooks_auth = {
+                n_guid: self.note_client.auth_linked_notebook(n_guid)
+                for n_guid in linked_notebooks
+            }
+
     def _raise_on_wrong_user(self) -> None:
         remote_user = self.note_client.user
         local_user = self.storage.config.get_config_value("user")
@@ -125,7 +162,14 @@ class NoteSynchronizer(object):
         if remote_user != local_user:
             raise WrongAuthUserError(local_user, remote_user)
 
-    def _sync_chunks(self, current_usn: int, remote_usn: int) -> None:
+    def _sync_chunks(self) -> None:
+        remote_usn = self.note_client.get_remote_usn()
+        current_usn = int(self.storage.config.get_config_value("USN"))
+
+        if remote_usn == current_usn:
+            logger.info("User notebooks are up to date, nothing to sync!")
+            return
+
         last_usn = current_usn
         with progressbar(
             length=remote_usn - current_usn,
@@ -133,24 +177,52 @@ class NoteSynchronizer(object):
             file=get_progress_output(),
         ) as chunks_bar:
             for chunk in self.note_client.iter_sync_chunks(current_usn):
-                if current_usn > 0:
-                    self._expunge(chunk.expungedNotebooks, chunk.expungedNotes)
-
-                if chunk.notebooks:
-                    self.storage.notebooks.add_notebooks(chunk.notebooks)
-
-                    self._count_updated_notebooks += len(chunk.notebooks)
-
-                if chunk.notes:
-                    self.storage.notes.add_notes_for_sync(chunk.notes)
+                self._process_chunk(chunk)
 
                 self.storage.config.set_config_value("USN", str(chunk.chunkHighUSN))
 
                 chunks_bar.update(chunk.chunkHighUSN - last_usn)
                 last_usn = chunk.chunkHighUSN
 
+    def _sync_linked_notebook(self, l_notebook: LinkedNotebook) -> None:
+        current_usn = self.storage.notebooks.get_linked_notebook_usn(l_notebook.guid)
+
+        l_notebook_chunks = self.note_client.iter_linked_notebook_sync_chunks(
+            l_notebook, current_usn
+        )
+
+        for chunk in l_notebook_chunks:
+            for notebook in chunk.notebooks or []:
+                # Correct stack info is in LinkedNotebook
+                notebook.stack = l_notebook.stack
+                self.storage.notebooks.add_linked_notebook(l_notebook, notebook)
+
+            self._process_chunk(chunk)
+
+            self.storage.notebooks.set_linked_notebook_usn(
+                l_notebook.guid, chunk.chunkHighUSN
+            )
+
+    def _process_chunk(self, chunk: SyncChunk) -> None:
+        self._expunge(
+            chunk.expungedNotebooks,
+            chunk.expungedNotes,
+            chunk.expungedLinkedNotebooks,
+        )
+
+        if chunk.notebooks:
+            self.storage.notebooks.add_notebooks(chunk.notebooks)
+
+            self._count_updated_notebooks += len(chunk.notebooks)
+
+        if chunk.notes:
+            self.storage.notes.add_notes_for_sync(chunk.notes)
+
     def _expunge(
-        self, expunged_notebooks: List[str], expunged_notes: List[str]
+        self,
+        expunged_notebooks: List[str],
+        expunged_notes: List[str],
+        expunged_linked_notebooks: List[str],
     ) -> None:
         if expunged_notebooks:
             self.storage.notebooks.expunge_notebooks(expunged_notebooks)
@@ -162,10 +234,21 @@ class NoteSynchronizer(object):
 
             self._count_expunged_notes += len(expunged_notes)
 
-    def _download_scheduled_notes(
-        self, notes_to_sync: Tuple[Tuple[str, str], ...]
-    ) -> None:
+        if expunged_linked_notebooks:
+            for l_notebook_guid in expunged_linked_notebooks:
+                notebook = self.storage.notebooks.get_notebook_by_linked_guid(
+                    l_notebook_guid
+                )
 
+                if notebook:
+                    self.storage.notebooks.expunge_notebooks((notebook.guid,))
+                    self.storage.notes.expunge_notes_by_notebook(notebook.guid)
+
+            self.storage.notebooks.expunge_linked_notebooks(expunged_linked_notebooks)
+
+            self._count_expunged_linked_notebooks += len(expunged_linked_notebooks)
+
+    def _download_scheduled_notes(self, notes_to_sync: Tuple[NoteForSync, ...]) -> None:
         with ThreadPoolExecutor(max_workers=self.max_download_workers) as executor:
             with progressbar(
                 length=len(notes_to_sync),
@@ -182,11 +265,15 @@ class NoteSynchronizer(object):
         self,
         executor: Any,
         notes_bar: Any,
-        notes_chunk: Iterable[Tuple[str, str]],
+        notes_chunk: Iterable[NoteForSync],
     ) -> None:
         note_futures = {
-            executor.submit(self.note_worker, note_guid): note_title
-            for note_guid, note_title in notes_chunk
+            executor.submit(
+                self.note_worker,
+                n.guid,
+                self.linked_notebooks_auth.get(n.linked_notebook_guid),  # type: ignore
+            ): n.title
+            for n in notes_chunk
         }
 
         try:
